@@ -2,6 +2,7 @@ package cmd
 
 import (
 	"crypto/sha256"
+	"crypto/tls"
 	"crypto/x509"
 	"encoding/hex"
 	"errors"
@@ -96,10 +97,12 @@ type clientConfigObfs struct {
 }
 
 type clientConfigTLS struct {
-	SNI       string `mapstructure:"sni"`
-	Insecure  bool   `mapstructure:"insecure"`
-	PinSHA256 string `mapstructure:"pinSHA256"`
-	CA        string `mapstructure:"ca"`
+	SNI               string `mapstructure:"sni"`
+	Insecure          bool   `mapstructure:"insecure"`
+	PinSHA256         string `mapstructure:"pinSHA256"`
+	CA                string `mapstructure:"ca"`
+	ClientCertificate string `mapstructure:"clientCertificate"`
+	ClientKey         string `mapstructure:"clientKey"`
 }
 
 type clientConfigQUIC struct {
@@ -294,6 +297,31 @@ func (c *clientConfig) fillTLSConfig(hyConfig *client.Config) error {
 		}
 		hyConfig.TLSConfig.RootCAs = cPool
 	}
+	if c.TLS.ClientCertificate != "" && c.TLS.ClientKey != "" {
+		certLoader := &utils.LocalCertificateLoader{
+			CertFile: c.TLS.ClientCertificate,
+			KeyFile:  c.TLS.ClientKey,
+		}
+		// Try loading the cert-key pair here to catch errors early
+		err := certLoader.InitializeCache()
+		if err != nil {
+			var pathErr *os.PathError
+			if errors.As(err, &pathErr) {
+				if pathErr.Path == c.TLS.ClientCertificate {
+					return configError{Field: "tls.clientCertificate", Err: pathErr}
+				}
+				if pathErr.Path == c.TLS.ClientKey {
+					return configError{Field: "tls.clientKey", Err: pathErr}
+				}
+			}
+			return configError{Field: "tls.clientCertificate", Err: err}
+		}
+		// Use GetClientCertificates so that users can update the cert without restarting the client.
+		hyConfig.TLSConfig.GetClientCertificate = func(*tls.CertificateRequestInfo) (*tls.Certificate, error) {
+			// For simplicity, always respond with the configured client certs, regardless of server requests.
+			return certLoader.GetCertificate(nil)
+		}
+	}
 	return nil
 }
 
